package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysTaskTimeExpressionService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysTaskTimeService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.StringTool;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ImppEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.*;
import cn.estsh.impp.framework.base.controller.CoreBaseController;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Description : 定时任务服务
 * @Reference   :
 * @Author      : wei.peng
 * @Date        : 2018-10-22 16:58:43.982
 * @Modify      :
 **/
@RestController
@Api(description = "定时任务服务")
@RequestMapping(CoreBaseController.BASE_URL +"/task-time")
public class SysTaskTimeController extends CoreBaseController{

    public static final Logger LOGGER = LoggerFactory.getLogger(SysTaskTimeController.class);

    @Autowired
    private ISysTaskTimeService taskTimeService;

    @Autowired
    private ISysTaskTimeExpressionService taskTimeExpressionService;

    @PostMapping(value = "/insert")
    @ApiOperation(value = "新增定时任务", notes = "定时任务")
    public ResultBean insertSysTaskTime(SysTaskTime taskTime,Boolean isExecute) {
        try {
            // 数据校验
            ValidatorBean.beginValid(taskTime)
                    .notNull("name", taskTime.getName())
                    .notNull("groupName",taskTime.getGroupName())
                    .notNull("timeExpressionId",taskTime.getTimeExpressionId())
                    .notNull("taskMethodType", taskTime.getTaskMethodType())
                    .notNull("taskPackage",taskTime.getTaskPackage())
                    .notNull("taskClass",taskTime.getTaskClass());

            SysTaskTimeExpression taskTimeExpression = taskTimeExpressionService
                    .getSysTaskTimeExpressionById(taskTime.getTimeExpressionId());
            ValidatorBean.checkNotNull(taskTimeExpression,"定时执行策略不存在");

            taskTime.setTimeExpressionNameRdd(taskTimeExpression.getName());
            taskTime.setTimeExpressionContentRdd(taskTimeExpression.getExpressionContent());
            taskTime.setTaskStartDateTimeRdd(taskTimeExpression.getExpressionStartDatetime());
            taskTime.setTaskEndDateTimeRdd(taskTimeExpression.getExpressionEndDatetime());
            taskTime.setTaskStatus(CommonEnumUtil.DATA_STATUS.ENABLE.getValue());

            taskTime  = taskTimeService.insertSysTaskTime(taskTime,taskTimeExpression);
            // 立即执行
            if(isExecute != null && isExecute){
                taskTimeService.executeSysTaskTimeById(taskTime.getId());
            }
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultObject(taskTime);
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @PutMapping(value = "/update")
    @ApiOperation(value = "修改定时任务", notes = "定时任务")
    public ResultBean updateSysTaskTime(SysTaskTime taskTime,Boolean isExecute) {
        try {
            // 数据校验
            ValidatorBean.beginValid(taskTime)
                    .notNull("id", taskTime.getId())
                    .notNull("name", taskTime.getName())
                    .notNull("groupName",taskTime.getGroupName())
                    .notNull("timeExpressionId",taskTime.getTimeExpressionId())
                    .notNull("taskMethodType", taskTime.getTaskMethodType())
                    .notNull("taskPackage",taskTime.getTaskPackage())
                    .notNull("taskClass",taskTime.getTaskClass());

            SysTaskTimeExpression taskTimeExpression = taskTimeExpressionService
                    .getSysTaskTimeExpressionById(taskTime.getTimeExpressionId());
            ValidatorBean.checkNotNull(taskTimeExpression,"定时执行策略不存在");

            taskTime.setTimeExpressionNameRdd(taskTimeExpression.getName());
            taskTime.setTimeExpressionContentRdd(taskTimeExpression.getExpressionContent());
            taskTime.setTaskStartDateTimeRdd(taskTimeExpression.getExpressionStartDatetime());
            taskTime.setTaskEndDateTimeRdd(taskTimeExpression.getExpressionEndDatetime());
            taskTime.setTaskStatus(CommonEnumUtil.DATA_STATUS.ENABLE.getValue());

            taskTimeService.updateSysTaskTime(taskTime,taskTimeExpression);
            if(isExecute != null && isExecute){
                taskTimeService.executeSysTaskTimeById(taskTime.getId());
            }
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultObject(taskTime);
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @DeleteMapping(value = "/delete/{id}")
    @ApiOperation(value = "删除定时任务", notes = "删除定时任务")
    public ResultBean deleteSysTaskTime(@PathVariable("id") String id) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "定时任务id 不能为空");

            taskTimeService.deleteSysTaskTimeById(Long.parseLong(id));
            return ResultBean.success().setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @GetMapping(value = "/list")
    @ApiOperation(value = "查询SysTaskTime", notes = "查询所有定时任务")
    public ResultBean findSysTaskTimeAll() {
        try {
            List<SysTaskTime> list = taskTimeService.listSysTaskTime();
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(list);
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @GetMapping(value = "/get/{id}")
    @ApiOperation(value = "查询定时任务", notes = "查询根据ID查询定时任务")
    public ResultBean getBySysTaskTimeId(@PathVariable("id") String id) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "定时任务id 不能为空");

            SysTaskTime taskTime = taskTimeService.getSysTaskTimeById(Long.parseLong(id));

            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultObject(taskTime);
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @PutMapping(value = "/status")
    @ApiOperation(value = "修改定时任务状态", notes = "根据ID修改定时任务状态")
    public ResultBean updateQuartzStatus(String id,int status) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "功能id 不能为空");
            ValidatorBean.checkNotZero(status, "功能状态不能为空");

            taskTimeService.updateSysTaskTimeStatus(Long.parseLong(id),status,AuthUtil.getSessionUser());
            return new ResultBean(true);
        } catch(ImppBusiException busExcep){
            LOGGER.error(busExcep.getErrorMsg() + "：{}",busExcep.getErrorDetail(),busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        }catch(Exception e){
            LOGGER.error(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getDescription() + "：{}",e.getMessage(),e);
            return ResultBean.fail().setCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode());
        }
    }

    @GetMapping("/query")
    @ApiOperation(value = "定时任务复杂查询，分页，排序",notes = "定时任务复杂查询，分页，排序")
    public ResultBean querySysTaskTimeByPager(SysTaskTime taskTime, Pager pager){
        try {
            ListPager taskTimeListPager = taskTimeService.querySysTaskTimeByPager(taskTime,pager);
            return  ResultBean.success("查询成功").setListPager(taskTimeListPager).setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            LOGGER.error(busExcep.getErrorMsg() + "：{}",busExcep.getErrorDetail(),busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        }catch(Exception e){
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}",e.getMessage(),e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @DeleteMapping("/batch-delete")
    @ApiOperation(value = "根据id批量删除定时任务",notes = "根据id批量删除定时任务")
    public ResultBean deleteSysTaskTimeByIds(String[] ids){
        try{
            if(ids.length == 0){
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要删除的资源。")
                        .build();
            }

            taskTimeService.deleteSysTaskTimeByIds(StringTool.getArrayLong(ids));
            return ResultBean.success("删除成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            LOGGER.error(busExcep.getErrorMsg() + "：{}",busExcep.getErrorDetail(),busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        }catch(Exception e){
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}",e.getMessage(),e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @PutMapping(value = "/batch-status")
    @ApiOperation(value = "批量修改系统任务状态",notes = "批量修改系统任务状态")
    public ResultBean updateSysTaskTimeStatusByIds(String[] ids,Integer status){
        try {
            // 数据校验
            if(ids.length == 0){
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要删除的资源。")
                        .build();
            }
            ValidatorBean.checkNotZero(status,"状态不能为空");

            taskTimeService.updateSysTaskTimeStatusByIds(StringTool.getArrayLong(ids),status);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch(ImppBusiException busExcep){
            LOGGER.error(busExcep.getErrorMsg() + "：{}",busExcep.getErrorDetail(),busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        }catch(Exception e){
            LOGGER.error(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getDescription() + "：{}",e.getMessage(),e);
            return ResultBean.fail().setCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode());
        }
    }

    @GetMapping(value = "/execute/{id}")
    @ApiOperation(value = "立即执行", notes = "立即执行")
    public ResultBean executeBySysTaskTimeId(@PathVariable("id") String id) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "定时任务id 不能为空");

            taskTimeService.executeSysTaskTimeById(Long.parseLong(id));
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }
}