package cn.estsh.i3plus.core.apiservice.mq;

import cn.estsh.i3plus.core.api.iservice.busi.ISysMessageService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysUserService;
import cn.estsh.i3plus.core.apiservice.util.MailUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ImppEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysMessage;
import cn.estsh.i3plus.pojo.platform.bean.SysRefUserMessage;
import cn.estsh.i3plus.pojo.platform.bean.SysUser;
import com.rabbitmq.client.Channel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.Message;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * @Description : 邮件队列处理
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2018-11-15 22:15
 * @Modify:
 **/
@Component
public class MailQueueReceiver {
    private static final Logger LOGGER = LoggerFactory.getLogger(MailQueueReceiver.class);

    @Autowired
    ISysMessageService sysMessageService;
    @Autowired
    ISysUserService sysUserService;

    /**
     * 系统邮件处理队列
     * @param msg
     * @param channel
     * @param message
     */
//    @RabbitListener(queues = I3CoreQueueConfig.IMPP_MESSAGE_MAIL_QUEUE)
    public void processImppMail(SysRefUserMessage msg, Channel channel, Message message) {
        try {
            LOGGER.info("【MQ-IMPP_MESSAGE_QUEUE】数据接收成功：{}",msg);
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            msg.setReceiverTime(sdf.format(new Date()));
            sysMessageService.insertSysRefUserMessage(msg);

            SysMessage sysMessage = sysMessageService.getSysMessageById(msg.getMessageId());
            SysUser sysUser = sysUserService.getSysUserById(msg.getReceiverId());

            MailUtil mailUtil = new MailUtil();
            mailUtil.setSubject(sysMessage.getMessageTitle());
            mailUtil.setContentType(ImppEnumUtil.MESSAGE_CONTENT_TYPE.valueOfDescription(sysMessage.getMessageContentType()));
            mailUtil.setBody(sysMessage.getMessageContent());
            mailUtil.setTo(sysUser.getUserEmail());
            mailUtil.send();

            //信息已处理
            channel.basicAck(message.getMessageProperties().getDeliveryTag(),false);
        } catch (IOException e) {
            LOGGER.error("【MQ-IMPP_MESSAGE_QUEUE】处理出错：{}",e.getMessage(),e);
            //丢弃这条消息
            try {
                // 未成功处理，重新发送
                channel.basicNack(message.getMessageProperties().getDeliveryTag(),false,true);
            } catch (IOException e1) {
                e1.printStackTrace();
            }
        }
    }
}
