package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysLocaleLanguageService;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysLocaleLanguage;
import cn.estsh.i3plus.pojo.platform.repository.SysLocaleLanguageRepository;
import cn.estsh.i3plus.pojo.platform.repository.SysLocaleResourceRepository;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.base.service.IBaseService;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Description :系统语言
 * @Reference :
 * @Author : yunhao
 * @CreateDate : 2018-11-12 13:48
 * @Modify:
 **/
@Service
public class SysLocaleLanguageService implements ISysLocaleLanguageService {

    public static final Logger LOGGER = LoggerFactory.getLogger(SysLocaleLanguageService.class);

    @Autowired
    private SysLocaleLanguageRepository sysLocaleLanguageRDao;

    @Autowired
    private SysLocaleResourceRepository sysLocaleResourceRDao;

    @Override
    public SysLocaleLanguage insertSysLocaleLanguage(SysLocaleLanguage sysLocaleLanguage) {
        LOGGER.info("系统语言 SYS_LOCALE_LANGUAGE SysLocaleLanguage:{}",sysLocaleLanguage);
        int count = sysLocaleLanguageRDao.findByPropertyCount("languageCode",sysLocaleLanguage.getLanguageCode());
        if (count > 0){
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                    .setErrorDetail("语言代码已存在")
                    .setErrorSolution("请重新输入语言代码")
                    .build();
        }

        return sysLocaleLanguageRDao.insert(sysLocaleLanguage);
    }

    @Override
    public void updateSysLocaleLanguage(SysLocaleLanguage sysLocaleLanguage) {
        SysLocaleLanguage data = sysLocaleLanguageRDao.getById(sysLocaleLanguage.getId());

        String hqlPack = CoreHqlPack.packHqlSysLocaleLanguageCode(sysLocaleLanguage);
        int count = sysLocaleLanguageRDao.findByHqlWhereCount(hqlPack);
        if (count > 0){
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                    .setErrorDetail("语言代码已存在")
                    .setErrorSolution("请重新输入语言代码")
                    .build();
        }

        // 根据资源中的语言代码和名称
        sysLocaleResourceRDao.updateByProperties(
                "languageCode", data.getLanguageCode(),
                new String[]{"languageCode", "languageNameRdd"},
                new Object[]{sysLocaleLanguage.getLanguageCode(), sysLocaleLanguage.getLanguageName()}
        );

        sysLocaleLanguageRDao.update(sysLocaleLanguage);
    }

    @Override
    public void deleteSysLocaleLanguageById(Long id) {
        SysLocaleLanguage sysLocaleLanguage = sysLocaleLanguageRDao.getById(id);
        ValidatorBean.checkNotNull(sysLocaleLanguage,"需要删除的资源不存在");

        if(sysLocaleResourceRDao.isExitByProperty("languageCode",sysLocaleLanguage.getLanguageCode())){
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_EXIT.getCode())
                    .setErrorDetail("存在关联资源无法删除")
                    .build();
        }

        sysLocaleLanguageRDao.deleteById(id);
    }

    @Override
    public List listSysLocaleLanguage() {
        LOGGER.info("系统语言 SYS_LOCALE_LANGUAGE list");
        return sysLocaleLanguageRDao.list();
    }

    @Override
    public ListPager querySysLocaleLanguageByPager(SysLocaleLanguage sysLocaleLanguage, Pager pager) {
        LOGGER.info("系统语言 SYS_LOCALE_LANGUAGE sysLocaleLanguage:{},Pager:{}",sysLocaleLanguage,pager);
        if(sysLocaleLanguage == null) {
            pager = PagerHelper.getPager(pager, sysLocaleLanguageRDao.listCount());
            return new ListPager(sysLocaleLanguageRDao.listPager(pager),pager);
        }else {
            String hqlPack = CoreHqlPack.packHqlSysLocaleLanguage(sysLocaleLanguage);
            pager = PagerHelper.getPager(pager, sysLocaleLanguageRDao.findByHqlWhereCount(hqlPack));
            return new ListPager(sysLocaleLanguageRDao.findByHqlWherePage(hqlPack + sysLocaleLanguage.orderBy(),pager),pager);
        }
    }

    @Override
    public void deleteSysLocaleLanguageByIds(Long[] ids) {
        LOGGER.info("系统语言 SYS_LOCALE_LANGUAGE ids:{}",ids);
        sysLocaleLanguageRDao.deleteByIds(ids);
    }

    @Override
    public void updateSysLocaleLanguageDefaultById(Long ids) {
        LOGGER.info("系统语言 SYS_LOCALE_LANGUAGE ids:{}",ids);
        sysLocaleLanguageRDao.updateByHqlWhere("and 1=1","isDefault",CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());

        sysLocaleLanguageRDao.updateByProperties("id",ids,
                "isDefault",CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
    }

    @Override
    public SysLocaleLanguage getSysLocaleLanguageByCode(String code) {
        return sysLocaleLanguageRDao.getByProperty("languageCode",code);
    }
}
