package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysUserService;
import cn.estsh.i3plus.core.apiservice.dao.IUserPermissionDao;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.EncryptTool;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.shirotoken.UserToken;
import cn.estsh.i3plus.pojo.base.tool.HqlPack;
import cn.estsh.i3plus.pojo.platform.bean.*;
import cn.estsh.i3plus.pojo.platform.repository.*;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import org.apache.shiro.authc.AuthenticationException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description : 平台用户业务
 * @Reference :
 * @Author : wei.peng
 * @Date : 2018-10-25 17:45
 * @Modify :
 **/
@Service
public class SysUserService implements ISysUserService {

    public static final Logger LOGGER = LoggerFactory.getLogger(SysUserService.class);

    @Autowired
    private SysUserRepository sysUserRDao;

    @Autowired
    private SysUserInfoRepository sysUserInfoRDao;

    @Autowired
    private IUserPermissionDao userPermissionDao;

    @Autowired
    private SysOrganizeRepository organizeRDao;

    @Autowired
    private SysRoleRepository sysRoleRDao;

    @Autowired
    private SysRefUserRoleRepository refUserRoleRDao;

    @Autowired
    private SysRefUserDepartmentRepository refUserDepartmentRDao;

    @Autowired
    private SysRefUserPositionRepository refUserPositionRDao;

    @Autowired
    private SysFileRepository sysFileRDao;

    @Override
    public SessionUser queryUserLogin(String loginName, String password, String languageCode) throws AuthenticationException {
        LOGGER.debug("平台用户 SYS_USER loginName:{} \t password:{} \t languageCode:{}", loginName, password, languageCode);

        UserToken token = new UserToken(loginName,password,languageCode);
        SessionUser sessionUser = AuthUtil.login(token);
        SysUser user = this.getSysUserByLoginName(loginName);

        if(user == null){
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.LOGIN_USER_NAME_EXCEPTION.getCode())
                    .setErrorDetail("用户名不存在")
                    .setErrorSolution("请重新输入用户名")
                    .build();
        }

        packSessionUser(sessionUser, user, user.getUserType(), languageCode);
        AuthUtil.setSessionUser(sessionUser);
        refreshUserLoginInformation(user.getId());
        return sessionUser;
    }

    @Override
    public void updateSysUserStatus(Long id, int status, SessionUser user) {
        LOGGER.debug("平台用户 SYS_USER id:{} status:{} modifyUser:{}", id, status, user.getUserName());
        SysUser sysUser = sysUserRDao.getById(id);
        sysUser.setUserStatus(status);
        sysUser.setModifyUser(user.getUserName());
        sysUserRDao.update(sysUser);
    }

    @Override
    public void updateBatchSysUserStatus(Long[] ids, int status, SessionUser user) {
        LOGGER.debug("平台用户 SYS_USER DELETE By ids :{} status:{}, SessionUser :{}", ids,status,user);

        String where = CoreHqlPack.packHqlIds("id", ids);
        List<SysUser> list = sysUserRDao.findByHqlWhere(where);
        if(null != list && list.size() > 0){
            for (SysUser sysUser : list) {
                sysUser.setUserStatus(status);
                sysUser.setIsValid(CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
                ConvertBean.modelUpdate(sysUser,user);
            }
            sysUserRDao.saveAll(list);
        }
    }

    @Override
    public void updateSysUser(SysUser sysUser) throws Exception{
        LOGGER.debug("平台用户 SYS_USER SysUser :{}", sysUser);
        sysUserRDao.save(sysUser);
    }

    @Override
    public void updateSysUserPassword(Long userId, String password, String newPwd) throws Exception {
        LOGGER.debug("平台用户 SYS_USER Override:{} password:{} newPwd:{}", userId,password,newPwd);
        SysUser user = sysUserRDao.getById(userId);
        if(user != null){
            String md5 = EncryptTool.hexMD5(password);
            if(md5.equals(user.getUserLoginPassword())){
                user.setUserLoginPassword(EncryptTool.hexMD5(newPwd));
                sysUserRDao.save(user);
            }else {
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.LOGIN_USER_NAME_PASSWORD_UPDATE_EXCEPTION.getCode())
                        .setErrorDetail("旧密码错误")
                        .setErrorSolution("请重新输入")
                        .build();
            }
        }else {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                    .setErrorDetail("用户不存在")
                    .setErrorSolution("请重新操作")
                    .build();
        }
    }

    @Override
    public void refreshUserLoginInformation(Long userId) {
        SysUser user = sysUserRDao.getById(userId);
        SysUserInfo userInfo = sysUserInfoRDao.getById(user.getUserInfoId());

        user.setUserLoginLastDateTime(TimeTool.getNowTime(true));
        user.setUserLoginNum(user.getUserLoginNum() + 1);
        userInfo.setUserLoginLastDateTime(TimeTool.getNowTime(true));
        userInfo.setUserLoginNum(userInfo.getUserLoginNum() + 1);

        sysUserInfoRDao.save(userInfo);
        sysUserRDao.save(user);
    }

    @Override
    public SysUser insertSysUser(SysUser sysUser)throws Exception {
        LOGGER.debug("平台用户 SYS_USER SysUser:{}", sysUser);

        SysUser loginUser = getSysUserByLoginName(sysUser.getUserLoginName());
        if (null == loginUser) {
            // 密码加密 输入的密码不等于密码一致时修改密码
            String md5 = EncryptTool.hexMD5(sysUser.getUserLoginPassword());
            sysUser.setUserLoginPassword(md5);
            return sysUserRDao.insert(sysUser);
        } else {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.IO_EXCEPTION.getCode())
                    .setErrorDetail("用户名已存在")
                    .setErrorSolution("请重新输入用户名")
                    .build();
        }
    }

    @Override
    public void refreshSysRefUserRole(SysUser sysUser, Long[] roleIds, String userName) {
        LOGGER.debug("平台用户 SYS_USER SysUser:{}", sysUser);
        LOGGER.debug("平台用户 SYS_USER String[] :{}", roleIds);

        // 根据IDS 查询所有的角色信息
        String where = CoreHqlPack.packHqlIds("id", roleIds);
        List<SysRole> list = sysRoleRDao.findByHqlWhere(where);

        // 删除角色关系信息
        refUserRoleRDao.deleteByProperty("userId", sysUser.getId()); // 删除用户角色关系

        if(null != list && list.size() > 0){
            // 新增用户角色关系
            List<SysRefUserRole> refUserRoles = new ArrayList<>();
            List<String> roleNameList = new ArrayList<>();
            SysRefUserRole refUserRole = null;
            for (SysRole role : list) {
                roleNameList.add(role.getName());
                // 用户角色关系创建
                refUserRole = new SysRefUserRole();
                refUserRole.setRoleId(role.getId());
                refUserRole.setRoleNameRdd(role.getName());
                refUserRole.setUserId(sysUser.getId());
                refUserRole.setUserNameRdd(sysUser.getUserName());
                refUserRole.setCreateUser(userName);
                refUserRole.setCreateDatetime(TimeTool.getNowTime(true));
                refUserRole.setModifyUser(userName);
                refUserRole.setModifyDatetime(TimeTool.getNowTime(true));

                refUserRoles.add(refUserRole);
            }
            refUserRoleRDao.saveAll(refUserRoles);

            // 角色名称冗余
            sysUser.setRoleNameRdd(String.join(",", roleNameList));
            sysUser.setRoleId(list.get(0).getId());

            sysUserRDao.save(sysUser);
        }
    }

    @Override
    public void deleteSysUserById(Long id) {
        LOGGER.debug("平台用户 SYS_USER DELETE By id :{}", id);
        sysUserRDao.updateByProperties("id", id, "isValid", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());

        // 删除用户关系(物理删除)
        refUserRoleRDao.deleteByProperty("userId", id); // 删除用户角色关系
        refUserDepartmentRDao.deleteByProperty("userId", id); //删除用户部门关系
        refUserPositionRDao.deleteByProperty("userId", id); //删除用户部门关系
    }

    @Override
    public void deleteBatchSysUserById(Long[] ids,SessionUser user) {
        LOGGER.debug("平台用户 SYS_USER DELETE By ids :{}", ids);

        // 标记用户已删除
        String where = CoreHqlPack.packHqlIds("id", ids);
        List<SysUser> list = sysUserRDao.findByHqlWhere(where);
        if(null != list && list.size() > 0){
            for (SysUser sysUser : list) {
                sysUser.setIsValid(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
            }

            sysUserRDao.saveAll(list);
            refUserRoleRDao.deleteByPropertyIn("userId",ids);
        }
    }

    @Override
    public List<SysUser> list() {
        LOGGER.debug("平台用户 SYS_USER find All");
        return sysUserRDao.findByProperty("isValid", CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
    }

    @Override
    public ListPager<SysUser> querySysUserByPager(SysUser user, Pager pager) {
        LOGGER.debug("系统用户 SysUser find user :{} page :{}", user, pager);

        //生成hql查询语句
        String hqlPack = CoreHqlPack.packHqlSysUser(user);
        pager = PagerHelper.getPager(pager, sysUserRDao.findByHqlWhereCount(hqlPack));
        return new ListPager(sysUserRDao.findByHqlWherePage(hqlPack + user.orderBy(), pager), pager);
    }

    @Override
    public SysUser getSysUserById(Long id) {
        LOGGER.debug("平台用户 SYS_USER get By id :{}", id);
        return sysUserRDao.getById(id);
    }

    @Override
    public SysUserInfo getSysUserInfoById(Long id) {
        return sysUserInfoRDao.getByProperty("id",id);
    }

    @Override
    public SysUser getSysUserByLoginName(String loginName) {
        LOGGER.debug("平台用户 SYS_USER find By Login Name :{}", loginName);
        return sysUserRDao.getByProperty(new String[]{"userLoginName", "isValid"},
                new Object[]{loginName, CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()});
    }

    @Override
    public List<SysRefUserRole> findSysRefUserRoleByUserId(Long userId) {
        return refUserRoleRDao.findByProperty("userId",userId);
    }

    @Override
    public boolean checkLoginName(String userLoginName, long id) {
        int count;
        if(id > 0){
            StringBuffer sw = new StringBuffer();
            HqlPack.getStringEqualPackOr(userLoginName,"userLoginName",sw);
            HqlPack.getNumNOEqualPack(id,"id",sw);
            count = sysUserRDao.findByHqlWhereCount(sw.toString());

        }else{
            count = sysUserRDao.findByPropertyCount("userLoginName",userLoginName);
        }
        return count <= 0 ;
    }

    /**
     * 封装用户登陆信息
     *
     * @param sessionUser  当前会话信息
     * @param user         当前登录用户
     * @param userType     用户类型
     * @param languageCode 用户选择的语言
     * @return
     */
    public SessionUser packSessionUser(SessionUser sessionUser, SysUser user, Integer userType, String languageCode) {
        LOGGER.debug("平台用户 SYS_USER user id :{}", user.getId());
        SysUserInfo userInfo = sysUserInfoRDao.getById(user.getUserInfoId());

        packSysUserRole(sessionUser,user);      //封装用户角色信息
        packSysUserMenu(sessionUser,user);      //封装用户功能信息

        if(userInfo != null){
            packSysUserDepartment(sessionUser,userInfo);    //封装用户部门信息
            packSysUserPosition(sessionUser,userInfo);      //封装用户岗位信息
            packSysUserOrganize(sessionUser,userInfo);      //封装用户组织信息

            userInfo.setUser(user);
            sessionUser.setUserName(userInfo.getName());
            sessionUser.setUserCode(userInfo.getUserEmpNo());
            sessionUser.setUserId(userInfo.getId());
            sessionUser.setUserInfo(userInfo);
        }

        sessionUser.setUserType(CommonEnumUtil.USER_TYPE.valueOf(userType));
        sessionUser.setLanguageCode(languageCode);
        sessionUser.setUser(user);

        return sessionUser;
    }

    /**
     * 封装用户角色信息
     * @param sessionUser
     * @param user
     * @return
     */
    public void packSysUserRole(SessionUser sessionUser, SysUser user){
        List<SysRole> roleList = null;
        try {
            LOGGER.debug("平台用户 SYS_USER user id :{}", user.getUserInfoId());

            roleList = userPermissionDao.findSysRoleByUser(user);
        }catch (Exception e){
            LOGGER.error(" SysUser Peck Role information Error userId : {} Exception Message : {} " ,user.getUserInfoId(),e.getMessage());
            e.printStackTrace();
        }

        user.setRoleList(roleList);
        sessionUser.setUser(user);
    }

    /**
     * 封装用户 菜单信息
     * @param sessionUser
     * @param user
     * @return
     */
    public void packSysUserMenu(SessionUser sessionUser, SysUser user){
        List<SysMenu> list = null;
        try {
            LOGGER.debug("平台用户 SYS_USER user id :{}", user.getUserInfoId());

            list = userPermissionDao.findSysMenuByUser(user);
        }catch (Exception e){
            LOGGER.error(" SysUser Peck Menu information Error userId : {} Exception Message : {} " ,user.getUserInfoId(),e.getMessage());
            e.printStackTrace();
        }

        user.setMenuList(list);
        sessionUser.setUser(user);
    }

    /**
     * 封装用户 部门信息封装
     * @param sessionUser
     * @param userInfo
     * @return
     */
    public void packSysUserDepartment(SessionUser sessionUser, SysUserInfo userInfo){
        List<SysDepartment> list = null;
        try {
            LOGGER.debug("平台用户 SYS_USER user id :{}", userInfo.getId());

            list = userPermissionDao.findDepartmentByUser(userInfo);
        }catch (Exception e){
            LOGGER.error(" SysUser Peck Department information Error userId : {} Exception Message : {} " ,userInfo.getId(),e.getMessage());
            e.printStackTrace();
        }
        userInfo.setDepartmentList(list);
        sessionUser.setUserInfo(userInfo);
    }

    /**
     *  封装用户 岗位信息封装
     * @param sessionUser
     * @param userInfo
     * @return
     */
    public void packSysUserPosition(SessionUser sessionUser, SysUserInfo userInfo){
        List<SysPosition> list = null;
        try {
            LOGGER.debug("平台用户 SYS_USER user id :{}", userInfo.getId());

            list = userPermissionDao.findPositionByUser(userInfo);
        }catch (Exception e){
            LOGGER.error(" SysUser Peck Position information Error userId : {} Exception Message : {} " ,userInfo.getId(),e.getMessage());
            e.printStackTrace();
        }
        userInfo.setPositionList(list);
        sessionUser.setUserInfo(userInfo);
    }

    /**
     *  封装用户 组织信息封装
     * @param sessionUser
     * @param userInfo
     * @return
     */
    public void packSysUserOrganize(SessionUser sessionUser, SysUserInfo userInfo){
        SysOrganize organize = null;
        try {
            LOGGER.debug("平台用户 SYS_USER user id :{}", userInfo.getId());

            organize = organizeRDao.getById(userInfo.getOrganizeId());
        }catch (Exception e){
            LOGGER.error(" SysUser Peck Organize information Error userId : {} Exception Message : {} " ,userInfo.getId(),e.getMessage());
        }
        userInfo.setOrganize(organize);
        sessionUser.setUserInfo(userInfo);
    }



}
